<?php
/* --------------------------------------------------------------
   HubConfigurationAjaxController.inc.php 2017-02-25
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

use \HubPublic\Exceptions\CurlRequestException;
use \HubPublic\Http\CurlRequest;

/**
 * Class GambioHubConfigurationAjaxController
 *
 * This controller contains AJAX functionality related to the configuration pages of the Gambio Hub module.
 *
 * @category   System
 * @package    AdminHttpViewControllers
 * @extends    AdminHttpViewController
 */
class HubConfigurationAjaxController extends AdminHttpViewController
{
	/**
	 * Get the shop data.
	 *
	 * If a Hub Client Key exists it will be returned instead of the shop data so that the merchant
	 * data can be loaded from the customer portal.
	 *
	 * @return JsonHttpControllerResponse Returns the shop data.
	 */
	public function actionGetShopData()
	{
		/**
		 * @var CountryService  $countryService
		 * @var CustomerService $customerService
		 */
		$countryService  = StaticGXCoreLoader::getService('Country');
		$customerService = StaticGXCoreLoader::getService('Customer');
		
		$customerData = $customerService->getCustomerById(new IdType($_SESSION['customer_id']));
		
		// Authentication hash.
		$authHash = AuthHashCreator::create();
		$state = STORE_ZONE ? (string)$countryService->getCountryZoneById(new IdType(STORE_ZONE))->getName() : '';
		
		$response = [
			'gender'      => (string)$customerData->getGender(),
			'firstname'   => TRADER_FIRSTNAME,
			'lastname'    => TRADER_NAME,
			'street'      => TRADER_STREET,
			'housenumber' => TRADER_STREET_NUMBER,
			'postcode'    => TRADER_ZIPCODE,
			'city'        => TRADER_LOCATION,
			'state'       => $state,
			'country'     => (string)$countryService->getCountryById(new IdType(STORE_COUNTRY))->getName(),
			'company'     => COMPANY_NAME,
			'email'       => (string)$customerData->getEmail(),
			'telephone'   => TRADER_TEL,
			'shopname'    => STORE_NAME,
			'shopkey'     => GAMBIO_SHOP_KEY,
			'authhash'    => $authHash->asString(),
			'version'     => ltrim(gm_get_conf('INSTALLED_VERSION'), 'v'),
			'url'         => HTTP_SERVER . DIR_WS_CATALOG
		];
		
		try
		{
			$hubClientKeyConfiguration = MainFactory::create('HubClientKeyConfiguration');
			
			$hubClientKey = $hubClientKeyConfiguration->get();
			
			$response['clientkey'] = $hubClientKey;
		}
		catch(Exception $exception)
		{
			// The app will continue in demo mode.
		}
		
		return MainFactory::create('JsonHttpControllerResponse', $response);
	}
	
	
	/**
	 * Starts the creation of the session key.
	 *
	 * @return JsonHttpControllerResponse Returns the session key data.
	 */
	public function actionCreateSessionKey()
	{
		// Start a new gambio hub session.
		$serviceFactory         = MainFactory::create('HubServiceFactory');
		$sessionKeyService      = $serviceFactory->createHubSessionKeyService();
		$clientKeyConfiguration = MainFactory::create('HubClientKeyConfiguration');
		$curlRequest            = new CurlRequest();
		$logControl             = LogControl::get_instance();
		$sessionsApiClient      = MainFactory::create('HubSessionsApiClient', MODULE_PAYMENT_GAMBIO_HUB_URL,
		                                              $sessionKeyService, $clientKeyConfiguration, $curlRequest,
		                                              $logControl);
		$authHash               = AuthHashCreator::create();
		
		try
		{
			$response['gambio_hub_session_key'] = $sessionsApiClient->startSession($authHash);
		}
		catch(UnexpectedValueException $exception)
		{
			// Send error.
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		catch(CurlRequestException $exception)
		{
			// Send error.
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		
		return MainFactory::create('JsonHttpControllerResponse', $response);
	}
	
	
	/**
	 * Returns the Gambio Hub Translations for the frontend.
	 *
	 * @return JsonHttpControllerResponse Returns the translation data.
	 */
	public function actionGetTranslations()
	{
		try
		{
			$languageTextManager = MainFactory::create('LanguageTextManager', 'gambio_hub_account',
			                                           $_SESSION['languages_id']);
			$response            = $languageTextManager->get_section_array();
		}
		catch(Exception $exception)
		{
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		
		return MainFactory::create('JsonHttpControllerResponse', $response);
	}
	
	
	/**
	 * Returns all available order statuses and the default one.
	 *
	 * @return JsonHttpControllerResponse Returns the order statuses data.
	 */
	public function actionGetOrderStatuses()
	{
		// Response.
		$response = [];
		
		// Order status key-value array (ID is the key, and name is the value).
		$availableOrderStatuses = [];
		
		// CodeIgniter query builder.
		$queryBuilder = StaticGXCoreLoader::getDatabaseQueryBuilder();
		
		// Language ID.
		$languageId = (int)$_SESSION['languages_id'];
		
		// Order statuses fetch query.
		$orderStatuses = $queryBuilder->get_where('orders_status', ['language_id' => $languageId])->result_array();
		
		// Iterate over each query result row and append the ID and name to the order status array.
		foreach($orderStatuses as $orderStatus)
		{
			// Order status ID.
			$id = (int)$orderStatus['orders_status_id'];
			
			// Order status name.
			$name = (string)$orderStatus['orders_status_name'];
			
			// Append data to order status array.
			$availableOrderStatuses[$id] = $name;
		}
		
		// Append data to response.
		$response['default'] = (int)DEFAULT_ORDERS_STATUS_ID;
		$response['all']     = $availableOrderStatuses;
		
		return MainFactory::create('JsonHttpControllerResponse', $response);
	}
	
	
	/**
	 * Updates Hub payment modules configuration.
	 *
	 * Payment module configuration values are necessary in various shop sections and therefore this method will
	 * update them.
	 *
	 * Provide a JSON string in the request body with these values:
	 *
	 *   - moduleCode >> Contains the module code to be updated.
	 *   - moduleConfiguration >> Contains the configuration key-value data.
	 *
	 * This method can be extended further for more configuration values.
	 */
	public function actionUpdateHubModuleConfiguration()
	{
		try
		{
			$body = json_decode(file_get_contents('php://input'), true);
			
			$this->_updateHubPaymentModuleAlias($body['moduleCode'], $body['moduleConfiguration']['alias']);
			
			$response = ['success' => true];
		}
		catch(Exception $exception)
		{
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		
		return MainFactory::create('JsonHttpControllerResponse', $response);
	}
	
	
	/**
	 * Updates/Inserts alias value for Gambio Hub module.
	 *
	 * @param string $moduleCode Gambio Hub module code.
	 * @param string $alias      New module alias string.
	 */
	protected function _updateHubPaymentModuleAlias($moduleCode, $alias)
	{
		$configurationKey = 'MODULE_PAYMENT_GAMBIO_HUB_' . strtoupper($moduleCode) . '_ALIAS';
		
		gm_set_conf($configurationKey, $alias);
	}
}